@@ -7,7 +7,12 @@ from networkx import (
 )
 from numpy import array
 
-from examples.data import DISTANCES, TRAVEL_TIMES, TIME_WINDOWS_LOWER, TIME_WINDOWS_UPPER
+from examples.data import (
+    DISTANCES,
+    TRAVEL_TIMES,
+    TIME_WINDOWS_LOWER,
+    TIME_WINDOWS_UPPER,
+)
 
 from vrpy import VehicleRoutingProblem
 
@@ -31,8 +36,8 @@ G = relabel_nodes(G, {0: "Source", 17: "Sink"})
 
 if __name__ == "__main__":
 
-    prob = VehicleRoutingProblem(G, time_windows=True)
-    prob.solve()
+    prob = VehicleRoutingProblem(G, time_windows=True, load_capacity=20)
+    prob.solve(cspy=False)
     print(prob.best_value)
     print(prob.best_routes)
     print(prob.arrival_time)
