@@ -1,4 +1,4 @@
-class MasterProblemBase:
+class _MasterProblemBase:
     """Base class for the master problems.
 
     Args:
@@ -7,7 +7,8 @@ class MasterProblemBase:
         routes (list): Current routes/variables/columns.
         drop_penalty (int, optional) : Value of penalty if node is dropped. Defaults to None.
         num_vehicles (int, optional): Maximum number of vehicles. Defaults to None.
-        periodic (bool, optional); True if vertices are to be visited periodically. Defaults to False.
+        periodic (bool, optional): True if vertices are to be visited periodically. Defaults to False.
+        minimize_global_span (bool, optional): True if global span (maximum distance) is minimized. Defaults to False.
         relax (bool, optional): True if variables are continuous. Defaults to True.
     """
 
@@ -19,9 +20,8 @@ class MasterProblemBase:
         drop_penalty,
         num_vehicles,
         periodic,
+        minimize_global_span,
         solver,
-        time_limit,
-        relax,
     ):
         self.G = G
         self.routes_with_node = routes_with_node
@@ -29,7 +29,5 @@ class MasterProblemBase:
         self.drop_penalty = drop_penalty
         self.num_vehicles = num_vehicles
         self.periodic = periodic
+        self.minimize_global_span = minimize_global_span
         self.solver = solver
-        self.time_limit = time_limit
-        self.relax = relax
-        self._tabu_list = []
