@@ -1,10 +1,7 @@
 import logging
 from time import time
-from pathlib import Path
 from typing import List, Union
-
 from networkx import DiGraph, shortest_path  # draw_networkx
-
 from vrpy.greedy import _Greedy
 from vrpy.master_solve_pulp import _MasterSolvePulp
 from vrpy.subproblem_lp import _SubProblemLP
@@ -18,10 +15,11 @@ from vrpy.checks import (
     check_feasibility,
     check_initial_routes,
     check_vrp,
+    check_pickup_delivery_time_windows,
+    check_periodic_num_vehicles,
 )
 from vrpy.preprocessing import get_num_stops_upper_bound
 from vrpy.hyper_heuristic import _HyperHeuristic
-from csv import DictWriter
 
 logger = logging.getLogger(__name__)
 
@@ -100,6 +98,7 @@ class VehicleRoutingProblem:
         self.drop_penalty = drop_penalty
         self.fixed_cost = fixed_cost
         self.num_vehicles = num_vehicles if num_vehicles is not None else []
+        self._num_vehicles_schedule = None
         self.periodic = periodic
         self.mixed_fleet = mixed_fleet
         self.minimize_global_span = minimize_global_span
@@ -147,7 +146,7 @@ class VehicleRoutingProblem:
         preassignments=None,
         pricing_strategy="BestEdges1",
         cspy=True,
-        exact=True,
+        exact=False,
         time_limit=None,
         solver="cbc",
         dive=False,
@@ -182,7 +181,7 @@ class VehicleRoutingProblem:
                 True if only cspy's exact algorithm is used to generate columns.
                 Otherwise, heuristics will be used until they produce +ve
                 reduced cost columns, after which the exact algorithm is used.
-                Defaults to True.
+                Defaults to False.
             time_limit (int, optional):
                 Maximum number of seconds allowed for solving (for finding columns).
                 Defaults to None.
@@ -265,23 +264,25 @@ class VehicleRoutingProblem:
         """Returns dict with route ids as keys and route costs as values."""
         cost = {}
         for route in self.best_routes:
-            edges = list(
-                zip(self.best_routes[route][:-1], self.best_routes[route][1:]))
+            edges = list(zip(self.best_routes[route][:-1], self.best_routes[route][1:]))
             k = self._best_routes_vehicle_type[route]
-            cost[route] = sum(
-                self._H.edges[i, j]["cost"][k] for (i, j) in edges)
+            cost[route] = sum(self._H.edges[i, j]["cost"][k] for (i, j) in edges)
         return cost
 
     @property
     def best_routes_load(self):
         """Returns dict with route ids as keys and route loads as values."""
         load = {}
-        if (not self.load_capacity or self.distribution_collection or
-                self.pickup_delivery):
+        if (
+            not self.load_capacity
+            or self.distribution_collection
+            or self.pickup_delivery
+        ):
             return load
         for route in self.best_routes:
             load[route] = sum(
-                self._H.nodes[v]["demand"] for v in self.best_routes[route])
+                self._H.nodes[v]["demand"] for v in self.best_routes[route]
+            )
         return load
 
     @property
@@ -291,20 +292,29 @@ class VehicleRoutingProblem:
         First key : route id ; second key : node ; value : load.
         If truck is collecting, load refers to accumulated load on truck.
         If truck is distributing, load refers to accumulated amount that has been unloaded.
+        If simultaneous distribution and collection, load refers to truck load when leaving the node.
         """
         load = {}
-        if (not self.load_capacity and not self.pickup_delivery and
-                not self.distribution_collection):
+        if (
+            not self.load_capacity
+            and not self.pickup_delivery
+            and not self.distribution_collection
+        ):
             return load
         for i in self.best_routes:
             load[i] = {}
             amount = 0
+            if self.distribution_collection:
+                k = self._best_routes_vehicle_type[i]
+                amount += self.load_capacity[k]
             for v in self.best_routes[i]:
                 amount += self._H.nodes[v]["demand"]
                 if self.distribution_collection:
-                    amount -= self._H.nodes[v]["collect"]
+                    # in this case, load = Q + collect - demand
+                    amount += (
+                        self._H.nodes[v]["collect"] - 2 * self._H.nodes[v]["demand"]
+                    )
                 load[i][v] = amount
-            del load[i]["Source"]
         return load
 
     @property
@@ -314,14 +324,13 @@ class VehicleRoutingProblem:
         if not self.duration and not self.time_windows:
             return duration
         for route in self.best_routes:
-            edges = list(
-                zip(self.best_routes[route][:-1], self.best_routes[route][1:]))
+            edges = list(zip(self.best_routes[route][:-1], self.best_routes[route][1:]))
             # Travel times
-            duration[route] = sum(
-                self._H.edges[i, j]["time"] for (i, j) in edges)
+            duration[route] = sum(self._H.edges[i, j]["time"] for (i, j) in edges)
             # Service times
-            duration[route] += sum(self._H.nodes[v]["service_time"]
-                                   for v in self.best_routes[route])
+            duration[route] += sum(
+                self._H.nodes[v]["service_time"] for v in self.best_routes[route]
+            )
 
         return duration
 
@@ -342,8 +351,9 @@ class VehicleRoutingProblem:
                 tail = route[j - 1]
                 head = route[j]
                 arrival[i][head] = max(
-                    arrival[i][tail] + self._H.nodes[tail]["service_time"] +
-                    self._H.edges[tail, head]["time"],
+                    arrival[i][tail]
+                    + self._H.nodes[tail]["service_time"]
+                    + self._H.edges[tail, head]["time"],
                     self._H.nodes[head]["lower"],
                 )
             del arrival[i]["Source"]
@@ -365,11 +375,15 @@ class VehicleRoutingProblem:
             for j in range(1, len(route) - 1):
                 tail = route[j - 1]
                 head = route[j]
-                departure[i][head] = (max(
-                    departure[i][tail] + self._H.nodes[tail]["service_time"] +
-                    self._H.edges[tail, head]["time"],
-                    self._H.nodes[head]["lower"],
-                ) + self._H.nodes[head]["service_time"])
+                departure[i][head] = (
+                    max(
+                        departure[i][tail]
+                        + self._H.nodes[tail]["service_time"]
+                        + self._H.edges[tail, head]["time"],
+                        self._H.nodes[head]["lower"],
+                    )
+                    + self._H.nodes[head]["service_time"]
+                )
         return departure
 
     @property
@@ -412,9 +426,12 @@ class VehicleRoutingProblem:
             G=self.G,
         )
         # Check feasibility
-        check_feasibility(load_capacity=self.load_capacity,
-                          G=self.G,
-                          duration=self.duration)
+        check_feasibility(
+            load_capacity=self.load_capacity, G=self.G, duration=self.duration
+        )
+        self.num_vehicles, self._num_vehicles_schedule = check_periodic_num_vehicles(
+            periodic=self.periodic, num_vehicles=self.num_vehicles
+        )
         # Lock preassigned routes
         if self._preassignments:
             self._lock()
@@ -422,9 +439,9 @@ class VehicleRoutingProblem:
         self._prune_graph()
         # Compute upper bound on number of stops as knapsack problem
         if self.load_capacity and not self.pickup_delivery:
-            num_stops = get_num_stops_upper_bound(self.G, self._max_capacity,
-                                                  self.num_stops,
-                                                  self.distribution_collection)
+            num_stops = get_num_stops_upper_bound(
+                self.G, self._max_capacity, self.num_stops, self.distribution_collection
+            )
             self.num_stops = num_stops
             logger.info("new upper bound : max num stops = %s" % num_stops)
 
@@ -463,7 +480,8 @@ class VehicleRoutingProblem:
         elif len(self.G.nodes()) > 2:
             # Solve as MIP
             _, _ = self.masterproblem.solve(
-                relax=False, time_limit=self._get_time_remaining(mip=True))
+                relax=False, time_limit=self._get_time_remaining(mip=True)
+            )
             (
                 self._best_value,
                 self._best_routes_as_graphs,
@@ -475,13 +493,16 @@ class VehicleRoutingProblem:
             # Generate good columns
             self._find_columns()
             # Stop if time limit is passed
-            if (isinstance(self._get_time_remaining(), float) and
-                    self._get_time_remaining() == 0.0):
+            if (
+                isinstance(self._get_time_remaining(), float)
+                and self._get_time_remaining() == 0.0
+            ):
                 logger.info("time up !")
                 break
             # Stop if no improvement limit is passed or max iter exceeded
             if self._no_improvement > 1000 or (
-                    self._max_iter and self._iteration >= self._max_iter):
+                self._max_iter and self._iteration >= self._max_iter
+            ):
                 break
 
     def _find_columns(self):
@@ -489,38 +510,46 @@ class VehicleRoutingProblem:
         # Solve restricted relaxed master problem
         if self._dive:
             duals, relaxed_cost = self.masterproblem.solve_and_dive(
-                time_limit=self._get_time_remaining())
+                time_limit=self._get_time_remaining()
+            )
             if self.hyper_heuristic:
                 self.hyper_heuristic.init(relaxed_cost)
         else:
             duals, relaxed_cost = self.masterproblem.solve(
-                relax=True, time_limit=self._get_time_remaining())
+                relax=True, time_limit=self._get_time_remaining()
+            )
         logger.info("iteration %s, %.6s" % (self._iteration, relaxed_cost))
         pricing_strategy = self._get_next_pricing_strategy(relaxed_cost)
 
         # One subproblem per vehicle type
         for vehicle in range(self._vehicle_types):
             # Solve pricing problem with randomised greedy algorithm
-            if (self._greedy and not self.time_windows and
-                    not self.distribution_collection and
-                    not self.pickup_delivery and not self.minimize_global_span):
+            if (
+                self._greedy
+                and not self.time_windows
+                and not self.distribution_collection
+                and not self.pickup_delivery
+                and not self.minimize_global_span
+            ):
                 subproblem = self._def_subproblem(duals, vehicle, greedy=True)
                 self.routes, self._more_routes = subproblem.solve(n_runs=20)
                 # Add initial_routes
                 if self._more_routes:
-                    for r in (r for r in self.routes
-                              if r.graph["name"] not in self.masterproblem.y):
+                    for r in (
+                        r
+                        for r in self.routes
+                        if r.graph["name"] not in self.masterproblem.y
+                    ):
                         self.masterproblem.update(r)
 
             # Continue searching for columns
             self._more_routes = False
             if not self.minimize_global_span:
                 self._more_routes = self._solve_subproblem_with_heuristic(
-                    pricing_strategy=pricing_strategy,
-                    vehicle=vehicle,
-                    duals=duals)
+                    pricing_strategy=pricing_strategy, vehicle=vehicle, duals=duals
+                )
             else:
-                for route in self._routes[:len(self._routes)]:
+                for route in self._routes[: len(self._routes)]:
                     self._more_routes = self._solve_subproblem_with_heuristic(
                         pricing_strategy=pricing_strategy,
                         vehicle=vehicle,
@@ -553,44 +582,43 @@ class VehicleRoutingProblem:
         duals=None,
         route=None,
     ):
-        """Solves pricing problem with input heuristic
-        """
+        """Solves pricing problem with input heuristic"""
         more_columns = False
         if self._pricing_strategy == "Hyper":
             if pricing_strategy == "BestPaths":
-                more_columns = self._attempt_solve_best_paths(vehicle=vehicle,
-                                                              duals=duals,
-                                                              route=route)
+                more_columns = self._attempt_solve_best_paths(
+                    vehicle=vehicle, duals=duals, route=route
+                )
             elif pricing_strategy == "BestEdges1":
-                more_columns = self._attempt_solve_best_edges1(vehicle=vehicle,
-                                                               duals=duals,
-                                                               route=route)
+                more_columns = self._attempt_solve_best_edges1(
+                    vehicle=vehicle, duals=duals, route=route
+                )
             elif pricing_strategy == "BestEdges2":
-                more_columns = self._attempt_solve_best_edges2(vehicle=vehicle,
-                                                               duals=duals,
-                                                               route=route)
+                more_columns = self._attempt_solve_best_edges2(
+                    vehicle=vehicle, duals=duals, route=route
+                )
             elif pricing_strategy == "Exact":
-                more_columns = self._attempt_solve_exact(vehicle=vehicle,
-                                                         duals=duals,
-                                                         route=route)
+                more_columns = self._attempt_solve_exact(
+                    vehicle=vehicle, duals=duals, route=route
+                )
         # old approach
         else:
             if pricing_strategy == "BestPaths":
-                more_columns = self._attempt_solve_best_paths(vehicle=vehicle,
-                                                              duals=duals,
-                                                              route=route)
+                more_columns = self._attempt_solve_best_paths(
+                    vehicle=vehicle, duals=duals, route=route
+                )
             elif pricing_strategy == "BestEdges1":
-                more_columns = self._attempt_solve_best_edges1(vehicle=vehicle,
-                                                               duals=duals,
-                                                               route=route)
+                more_columns = self._attempt_solve_best_edges1(
+                    vehicle=vehicle, duals=duals, route=route
+                )
             elif pricing_strategy == "BestEdges2":
-                more_columns = self._attempt_solve_best_edges2(vehicle=vehicle,
-                                                               duals=duals,
-                                                               route=route)
+                more_columns = self._attempt_solve_best_edges2(
+                    vehicle=vehicle, duals=duals, route=route
+                )
             if pricing_strategy == "Exact" or not more_columns:
-                more_columns = self._attempt_solve_exact(vehicle=vehicle,
-                                                         duals=duals,
-                                                         route=route)
+                more_columns = self._attempt_solve_exact(
+                    vehicle=vehicle, duals=duals, route=route
+                )
         return more_columns
 
     def _attempt_solve_best_paths(self, vehicle=None, duals=None, route=None):
@@ -604,7 +632,8 @@ class VehicleRoutingProblem:
                 k_shortest_paths,
             )
             self.routes, self._more_routes = subproblem.solve(
-                self._get_time_remaining())
+                self._get_time_remaining()
+            )
             more_columns = self._more_routes
             if more_columns:
                 break
@@ -623,7 +652,8 @@ class VehicleRoutingProblem:
                 alpha,
             )
             self.routes, self._more_routes = subproblem.solve(
-                self._get_time_remaining(),)
+                self._get_time_remaining(),
+            )
             more_columns = self._more_routes
             if more_columns:
                 break
@@ -655,15 +685,16 @@ class VehicleRoutingProblem:
 
     def _attempt_solve_exact(self, vehicle=None, duals=None, route=None):
         subproblem = self._def_subproblem(duals, vehicle, route)
-        self.routes, self._more_routes = subproblem.solve(
-            self._get_time_remaining())
+        self.routes, self._more_routes = subproblem.solve(self._get_time_remaining())
         return self._more_routes
 
     def _get_next_pricing_strategy(self, relaxed_cost):
         "Return the appropriate pricing strategy based on input parameters"
         pricing_strategy = None
-        if (self._pricing_strategy == "Hyper" and
-                self._no_improvement != self._run_exact):
+        if (
+            self._pricing_strategy == "Hyper"
+            and self._no_improvement != self._run_exact
+        ):
             self._no_improvement_iteration = self._iteration
             if self._iteration == 0:
                 pricing_strategy = "BestPaths"
@@ -680,16 +711,16 @@ class VehicleRoutingProblem:
         return pricing_strategy
 
     def _update_hyper_heuristic(self, relaxed_cost: float):
-        best_paths, best_paths_freq = self.masterproblem.get_heuristic_distribution(
-        )
+        best_paths, best_paths_freq = self.masterproblem.get_heuristic_distribution()
         self.hyper_heuristic.current_performance(
             new_objective_value=relaxed_cost,
             produced_column=self._more_routes,
-            active_columns=best_paths_freq)
+            active_columns=best_paths_freq,
+        )
         self.hyper_heuristic.move_acceptance()
-        self.hyper_heuristic.update_parameters(self._iteration,
-                                               self._no_improvement,
-                                               self._no_improvement_iteration)
+        self.hyper_heuristic.update_parameters(
+            self._iteration, self._no_improvement, self._no_improvement_iteration
+        )
 
     def _get_time_remaining(self, mip: bool = False):
         """
@@ -788,9 +819,13 @@ class VehicleRoutingProblem:
         """
         self._initial_routes = []
         # Run Clarke & Wright if possible
-        if (not self.time_windows and not self.pickup_delivery and
-                not self.distribution_collection and not self.mixed_fleet and
-                not self.periodic):
+        if (
+            not self.time_windows
+            and not self.pickup_delivery
+            and not self.distribution_collection
+            and not self.mixed_fleet
+            and not self.periodic
+        ):
             best_value = 1e10
             best_num_vehicles = 1e10
             for alpha in [x / 10 for x in range(1, 20)]:
@@ -811,23 +846,33 @@ class VehicleRoutingProblem:
                     best_value = alg.best_value
                     best_num_vehicles = len(alg.best_routes)
             logger.info(
-                "Clarke & Wright solution found with value %s and %s vehicles" %
-                (best_value, best_num_vehicles))
+                "Clarke & Wright solution found with value %s and %s vehicles"
+                % (best_value, best_num_vehicles)
+            )
 
             # Run greedy algorithm if possible
-            alg = _Greedy(self.G, self.load_capacity, self.num_stops,
-                          self.duration)
+            alg = _Greedy(self.G, self.load_capacity, self.num_stops, self.duration)
             alg.run()
-            logger.info("Greedy solution found with value %s and %s vehicles" %
-                        (alg.best_value, len(alg.best_routes)))
+            logger.info(
+                "Greedy solution found with value %s and %s vehicles"
+                % (alg.best_value, len(alg.best_routes))
+            )
             self._initial_routes += alg.best_routes
 
         # If pickup and delivery, initial routes are Source-pickup-delivery-Sink
         elif self.pickup_delivery:
             for v in self.G.nodes():
                 if "request" in self.G.nodes[v]:
-                    self._initial_routes.append(
-                        ["Source", v, self.G.nodes[v]["request"], "Sink"])
+                    obvious_route = ["Source", v, self.G.nodes[v]["request"], "Sink"]
+                    edges = [
+                        ("Source", v),
+                        (v, self.G.nodes[v]["request"]),
+                        (self.G.nodes[v]["request"], "Sink"),
+                    ]
+                    if self.time_windows:
+                        check_pickup_delivery_time_windows(self.G, edges)
+                    self._initial_routes.append(obvious_route)
+
         # Otherwise compute round trips
         else:
             alg = _RoundTrip(self.G)
@@ -890,8 +935,10 @@ class VehicleRoutingProblem:
     def _remove_infeasible_arcs_capacities(self):
         infeasible_arcs = []
         for (i, j) in self.G.edges():
-            if (self.G.nodes[i]["demand"] + self.G.nodes[j]["demand"] >
-                    self._max_capacity):
+            if (
+                self.G.nodes[i]["demand"] + self.G.nodes[j]["demand"]
+                > self._max_capacity
+            ):
                 infeasible_arcs.append((i, j))
         self.G.remove_edges_from(infeasible_arcs)
 
@@ -910,14 +957,13 @@ class VehicleRoutingProblem:
                     # earliest time is coming straight from depot
                     self.G.nodes[v]["lower"] = max(
                         self.G.nodes[v]["lower"],
-                        self.G.nodes["Source"]["lower"] +
-                        self.G.edges["Source", v]["time"],
+                        self.G.nodes["Source"]["lower"]
+                        + self.G.edges["Source", v]["time"],
                     )
                     # Latest time is going straight to depot
                     self.G.nodes[v]["upper"] = min(
                         self.G.nodes[v]["upper"],
-                        self.G.nodes["Sink"]["upper"] -
-                        self.G.edges[v, "Sink"]["time"],
+                        self.G.nodes["Sink"]["upper"] - self.G.edges[v, "Sink"]["time"],
                     )
         self.G.remove_edges_from(infeasible_arcs)
 
@@ -944,18 +990,19 @@ class VehicleRoutingProblem:
 
         for v in self.G.nodes():
             for attribute in [
-                    "demand",
-                    "collect",
-                    "service_time",
-                    "lower",
-                    "upper",
+                "demand",
+                "collect",
+                "service_time",
+                "lower",
+                "upper",
             ]:
                 if attribute not in self.G.nodes[v]:
                     self.G.nodes[v][attribute] = 0
             # Ignore demand at Source/Sink
             if v in ["Source", "Sink"] and self.G.nodes[v]["demand"] > 0:
-                logger.warning("Demand %s at node %s is ignored." %
-                               (self.G.nodes[v]["demand"], v))
+                logger.warning(
+                    "Demand %s at node %s is ignored." % (self.G.nodes[v]["demand"], v)
+                )
                 self.G.nodes[v]["demand"] = 0
 
             # Set frequency = 1 if missing
@@ -975,9 +1022,11 @@ class VehicleRoutingProblem:
 
         if self.G.nodes["Sink"]["upper"] == 0:
             self.G.nodes["Sink"]["upper"] = max(
-                self.G.nodes[u]["upper"] + self.G.nodes[u]["service_time"] +
-                self.G.edges[u, "Sink"]["time"]
-                for u in self.G.predecessors("Sink"))
+                self.G.nodes[u]["upper"]
+                + self.G.nodes[u]["service_time"]
+                + self.G.edges[u, "Sink"]["time"]
+                for u in self.G.predecessors("Sink")
+            )
 
     def _update_dummy_attributes(self):
         """Adds dummy attributes on nodes and edges if missing."""
@@ -1006,8 +1055,7 @@ class VehicleRoutingProblem:
         for route in self._best_routes_as_graphs:
             node_list = shortest_path(route, "Source", "Sink")
             self._best_routes[route_id] = node_list
-            self._best_routes_vehicle_type[route_id] = route.graph[
-                "vehicle_type"]
+            self._best_routes_vehicle_type[route_id] = route.graph["vehicle_type"]
             route_id += 1
         # Merge with preassigned complete routes
         for route in self._preassignments:
@@ -1017,8 +1065,7 @@ class VehicleRoutingProblem:
                 best_cost = 1e10
                 for k in range(self._vehicle_types):
                     # If different vehicles, the cheapest feasible one is accounted for
-                    cost = sum(
-                        self._H.edges[i, j]["cost"][k] for (i, j) in edges)
+                    cost = sum(self._H.edges[i, j]["cost"][k] for (i, j) in edges)
                     load = sum(self._H.nodes[i]["demand"] for i in route)
                     if cost < best_cost:
                         if self.load_capacity:
@@ -1069,7 +1116,7 @@ class VehicleRoutingProblem:
                 self.periodic,
                 self.best_routes,
                 self.best_routes_type,
-                self.num_vehicles,
+                self._num_vehicles_schedule,
                 solver,
             )
             schedule.solve(self._get_time_remaining())
